/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.actions;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;

/**
 * The {@link DeleteAction} uses the {@link IElementCompositorService} and the
 * {@link IConnectionCompositorService} to determine the enabled state of this
 * action.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 4E9C8E039F44355D95C95D006A5369AD
 */
public class DeleteAction extends EObjectActionBase<EObject> {

	/** Constructor. */
	public DeleteAction() {
		super("Delete", PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		setId(ActionFactory.DELETE.getId());
	}

	/** {@inheritDoc} */
	@Override
	public void run() {
		doDelete(getTargets());
	}

	/** Deletes the given targets (organized in a map by topLevel element). */
	public static void doDelete(List<EObject> unorganizedTargets) {
		Map<ITopLevelElement, List<EObject>> topLevelTargetsMap = topLevelMap(unorganizedTargets);
		for(ITopLevelElement topLevel : topLevelTargetsMap.keySet()) {
			final List<EObject> targets = topLevelTargetsMap.get(topLevel);
			boolean topLevelDeleted = false;
			for(EObject target : targets) {
				if(IPersistencyService.INSTANCE.isTopLevelElement(target)) {
					doTopLevelElementDelete(target);
					topLevelDeleted = true;
				}
			}
			if(!topLevelDeleted) {
				ICommandStackService.INSTANCE.runAsCommand(topLevel.getRootModelElement(),
						new Runnable() {
							@Override
							public void run() {
								doCompositorBasedDelete(targets);
							}
						});
			}
		}

	}

	/** Computes enabled state of the delete action. */
	@Override
	protected boolean computeEnabled() {
		List<EObject> targets = getTargets();

		for(EObject target : targets) {
			if(canDelete(target)) {
				return true;
			}
		}
		return false;
	}

	/** Compute if one given target can be deleted. */
	private boolean canDelete(EObject target) {
		if(!IPersistencyService.INSTANCE.isTopLevelElement(target)) {
			// use composition services for deletion
			return(IElementCompositorService.INSTANCE.canDecompose(target) || IConnectionCompositorService.INSTANCE
					.canDisconnect(target));
		}
		// use top-level element deletion mechanism
		ITopLevelElement topElement = IPersistencyService.INSTANCE.getTopLevelElementFor(target);
		return topElement.canDelete();
	}

	/** Executes the delete using the correct service. */
	private static void doCompositorBasedDelete(List<EObject> targets) {
		for(EObject target : targets) {
			if(IElementCompositorService.INSTANCE.canDecompose(target)) {
				IElementCompositorService.INSTANCE.decompose(target);
			} else if(IConnectionCompositorService.INSTANCE.canDisconnect(target)) {
				IConnectionCompositorService.INSTANCE.disconnect(target);
			}
		}
	}

	/** User interaction before deleting a top-level element. */
	private static void doTopLevelElementDelete(EObject target) {
		ITopLevelElement topElement = IPersistencyService.INSTANCE.getTopLevelElementFor(target);
		Shell sh = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
		String message = "Are you sure you want to delete '" + topElement.getSaveableName() + "'?";

		final MessageDialog dialog =
				new MessageDialog(sh, "Confirm Delete", null, message, MessageDialog.QUESTION,
						new String[] {IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL}, 0) {
					@Override
					protected int getShellStyle() {
						return super.getShellStyle() | SWT.SHEET;
					}
				};

		sh.getDisplay().syncExec(new Runnable() {
			@Override
			public void run() {
				dialog.open();
			}
		});

		if(dialog.getReturnCode() == 0) {
			topElement.delete();
			// close also all opened views belonging to the deleted project
			IModelEditorBindingService.INSTANCE.closeEditors(topElement.getRootModelElement());
		}
	}

	/**
	 * Returns a map grouping the given model elements by top-level element,
	 * i.e., given a set of model elements M, returns a map f s.t., for every top level element top,
	 * f(top) = { m in M | top is the top level element of m }
	 */
	public static Map<ITopLevelElement, List<EObject>> topLevelMap(List<EObject> modelElements) {
		HashMap<ITopLevelElement, List<EObject>> res =
				new HashMap<ITopLevelElement, List<EObject>>();
		for(EObject eo : modelElements) {
			ITopLevelElement top = IPersistencyService.INSTANCE.getTopLevelElementFor(eo);
			List<EObject> elts = res.get(top);
			List<EObject> elts2;
			if(elts == null) {
				elts2 = Arrays.asList(eo);
			} else {
				elts2 = new ArrayList<EObject>(elts);
				elts2.add(eo);
			}
			res.put(top, elts2);
		}
		return res;
	}

}
